/*
 * ################################################################
 *
 * ProActive Parallel Suite(TM): The Java(TM) library for
 *    Parallel, Distributed, Multi-Core Computing for
 *    Enterprise Grids & Clouds
 *
 * Copyright (C) 1997-2012 INRIA/University of
 *                 Nice-Sophia Antipolis/ActiveEon
 * Contact: proactive@ow2.org or contact@activeeon.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; version 3 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package unitTests.gcmdeployment.commandEscape;

import static unitTests.UnitTests.logger;

import java.io.IOException;

import org.junit.Assert;
import org.junit.Test;
import org.objectweb.proactive.extensions.gcmdeployment.Helpers;


public class TestCommandEscape {
    final static String sshLocalhost = "ssh localhost";
    final static String cTrue = "true *";
    final static int vTrue = 0;
    final static String cFalse = "/bin/false \"plop\"";
    final static int vFalse = 1;

    @Test
    public void testCommandEscape() throws IOException, InterruptedException {
        String cmdT = cTrue;
        String cmdF = cFalse;

        logger.debug("Executing: " + cTrue);
        Assert.assertTrue(exec(cTrue) == vTrue);
        logger.debug("Executing: " + cFalse);
        Assert.assertTrue(exec(cFalse) == vFalse);

        cmdT = concat(sshLocalhost, cmdT);
        cmdF = concat(sshLocalhost, cmdF);

        logger.debug("Executing: " + cTrue);
        Assert.assertTrue(exec(cTrue) == vTrue);
        logger.debug("Executing: " + cFalse);
        Assert.assertTrue(exec(cFalse) == vFalse);

        cmdT = concat(sshLocalhost, cmdT);
        cmdF = concat(sshLocalhost, cmdF);

        logger.debug("Executing: " + cTrue);
        Assert.assertTrue(exec(cTrue) == vTrue);
        logger.debug("Executing: " + cFalse);
        Assert.assertTrue(exec(cFalse) == vFalse);

        cmdT = concat(sshLocalhost, cmdT);
        cmdF = concat(sshLocalhost, cmdF);

        logger.debug("Executing: " + cTrue);
        Assert.assertTrue(exec(cTrue) == vTrue);
        logger.debug("Executing: " + cFalse);
        Assert.assertTrue(exec(cFalse) == vFalse);
    }

    static private String concat(String prefixCmd, String cmd) {
        return prefixCmd + " " + Helpers.escapeCommand(cmd);
    }

    static private int exec(String cmd) throws IOException, InterruptedException {
        Process p = Runtime.getRuntime().exec(cmd);
        p.waitFor();
        return p.exitValue();
    }
}
